@@ -57,7 +57,7 @@ int addrEncode(char encodeAddr[], char dotSepAddr[])
     snprintf(encodeAddr, ADDRMAXLN,
 	     "%lu-%d-%d", address,checkDigit, (int)time(NULL));
   }else{
-    encodeAddr[0]='\0';
+    strlcpy(encodeAddr, "0-0-0", ADDRMAXLN);
   }
 
   return illform;
