@@ -25,6 +25,7 @@ Email: watanaby@is.saga-u.ac.jp
 Programmed by Yoshiaki WATANABE
 Modified by Shin-ichi TADAKI
 Modified by Katsuhiko Eguchi
+Modified by Makoto Otani
 **************************************************/
 
 #include	"opengatesrv.h"
@@ -41,7 +42,7 @@ void decode(char *string);
 /*******************************/
 void getClientAddr(char *clientAddr)
 {
-  strncpy(clientAddr, getenv("REMOTE_ADDR"), ADDRMAXLN);
+  strlcpy(clientAddr, getenv("REMOTE_ADDR"), ADDRMAXLN);
 }
 
 /********************************************/
@@ -86,17 +87,17 @@ int getPostData(char *userid, char *password, char *clientAddr4, int *durationPt
       split(ptr, name, value, next);
       
       if(strstr(name[0], "userid")!=NULL){
-	strncpy(userid, value[0], USERMAXLN);
+	strlcpy(userid, value[0], USERMAXLN);
       }else if(strstr(name[0], "password")!=NULL){
-	strncpy(password, value[0], PASSMAXLN);
+	strlcpy(password, value[0], PASSMAXLN);
       }else if(strstr(name[0],"remote_addr")!=NULL){
-	strncpy(encodeAddr4,value[0],ADDRMAXLN);
+	strlcpy(encodeAddr4,value[0],ADDRMAXLN);
       }else if(strstr(name[0], "language")!=NULL){
-	strncpy(language, value[0], WORDMAXLN);
+	strlcpy(language, value[0], WORDMAXLN);
       }else if(strstr(name[0], "duration")!=NULL){
-	strncpy(durationStr, value[0], WORDMAXLN);
+	strlcpy(durationStr, value[0], WORDMAXLN);
       }else if(strstr(name[0], "redirected_url")!=NULL){
-	strncpy(redirectedUrl, value[0], BUFFMAXLN);
+	strlcpy(redirectedUrl, value[0], BUFFMAXLN);
       }
       ptr=next[0];
     }
@@ -107,7 +108,7 @@ int getPostData(char *userid, char *password, char *clientAddr4, int *durationPt
      &&(queryStrLen=strlen(getenv("QUERY_STRING")))!=0){
 
     /* get html access parameter string */
-    strncpy(queryStr, getenv("QUERY_STRING"), BUFFMAXLN);
+    strlcpy(queryStr, getenv("QUERY_STRING"), BUFFMAXLN);
 
     /* split language and address in content
        [addr=0-0-0&lang=ja&redirectedurl=xxxx] */
@@ -115,11 +116,11 @@ int getPostData(char *userid, char *password, char *clientAddr4, int *durationPt
     while(ptr!=NULL){
       split(ptr, name, value, next);
       if(strstr(name[0], "addr")!=NULL){
-	strncpy(encodeAddr4,value[0],ADDRMAXLN);
+	strlcpy(encodeAddr4,value[0],ADDRMAXLN);
       }else if(strstr(name[0], "lang")!=NULL){
-	strncpy(language, value[0], WORDMAXLN);
+	strlcpy(language, value[0], WORDMAXLN);
       }else if(strstr(name[0], "redirectedurl")!=NULL){
-	strncpy(redirectedUrl, value[0], BUFFMAXLN);
+	strlcpy(redirectedUrl, value[0], BUFFMAXLN);
       }
       ptr=next[0];
     }
@@ -140,24 +141,16 @@ int getPostData(char *userid, char *password, char *clientAddr4, int *durationPt
   decode(redirectedUrl);
 
  /* if not available language, use first lang */
-  strncpy(langList, GetConfValue("HtmlLangs"), BUFFMAXLN);  /* list of available languages */
-  if(strstr(langList,language)==NULL){
+  strlcpy(langList, GetConfValue("HtmlLangs"), BUFFMAXLN);  /* list of available languages */
+  if(isNull(language) || strstr(langList,language)==NULL){
     sscanf(langList,"%s",language);
   }
   
   /* convert duration string to interger and minutes to seconds */
   *durationPtr = atoi(durationStr)*60;
-  *durationEntered = TRUE;
-
-  /* usage duration is restricted to permitted range */
-  if(*durationPtr <= 0){
-    *durationEntered = FALSE;
-    *durationPtr= atoi(GetConfValue("Duration/Default"));
-  }else{
-    int durmax=atoi(GetConfValue("Duration/Max"));
-    if(*durationPtr > durmax) *durationPtr=durmax;
-  }
-
+  if(*durationPtr > 0) *durationEntered = TRUE;
+  else *durationEntered = FALSE;
+  
   /* encoded address starting as "0-0-0" means no addr info */
   /* it indicates needless to get dual stack addresses */
   /* and only use getenv("REMOTE_ADDR") address */
@@ -173,10 +166,10 @@ int getPostData(char *userid, char *password, char *clientAddr4, int *durationPt
   }
 
   /* if the decoded IPv4 addr is not same as access IPv4 addr, use later */
-  strncpy(accessAddr, getenv("REMOTE_ADDR"), ADDRMAXLN);
+  strlcpy(accessAddr, getenv("REMOTE_ADDR"), ADDRMAXLN);
   if((strnstr(accessAddr, ".", ADDRMAXLN)!=NULL)   /* access is IPv4 */
      && strncmp(accessAddr, clientAddr4, ADDRMAXLN)!=0){  /* and not same */
-    strncpy(clientAddr4, accessAddr, ADDRMAXLN);
+    strlcpy(clientAddr4, accessAddr, ADDRMAXLN);
   }
 
   /* if no userid, set useid="?" */
@@ -189,10 +182,66 @@ int getPostData(char *userid, char *password, char *clientAddr4, int *durationPt
 }
 
 
+/********************************************/
+/* get userid from environment variable     */
+/********************************************/
+int getUserIdFromEnv(char *userid){
+
+  int ret=FALSE;
+  char* pEnv=NULL;
+
+  /* if shibboleth or httpbasic, get uid from environment var */
+  /* shibboleth */
+  if(strcmp(GetConfValue("AuthServer/Protocol"), "shibboleth")==0){
+    pEnv=getenvEx(GetConfValue("AuthServer/UidAttribute"),TRUE,TRUE);
+    if(!isNull(pEnv)){
+      strlcpy(userid, pEnv, USERMAXLN);
+
+      /* if idp string can be get from env variable, concatinate it as uid@org */
+      pEnv=getenvEx(GetConfValue("AuthServer/OrgAttribute"),TRUE,TRUE);
+      if(!isNull(pEnv)){
+	strlcat(userid, GetConfValue("UserIdSeparator"), USERMAXLN);
+	strlcat(userid, pEnv, USERMAXLN);
+      } 
+      ret=TRUE;
+    }
+
+    /* get from env-variable for EPPN(edu person principal name) attribute */
+    else{
+      pEnv=getenvEx(GetConfValue("AuthServer/EppnAttribute"),TRUE,FALSE);
+      if(!isNull(pEnv)){
+	strlcat(userid, pEnv, USERMAXLN);
+	ret=TRUE;
+      }
+    }
+
+    /* no env-variable is found */
+    if(ret==FALSE){
+      err_msg("ERR at %s#%d: Cannot get user info from shibboleth",__FILE__,__LINE__);
+      PutClientMsg("Cannot get user info from shibboleth<br>Check shibboleth setting in .htaccess and other");
+      exit(0);
+    }    
+  }
+
+  /* httpbasic */
+  else if(strcmp(GetConfValue("AuthServer/Protocol"), "httpbasic")==0){
+    if(!isNull(getenv("REMOTE_USER"))){
+      strlcpy(userid,getenv("REMOTE_USER"),USERMAXLN);
+      ret=TRUE;
+    }else{
+      err_msg("ERR at %s#%d: Cannot get user info from httpbasic",__FILE__,__LINE__);
+      ret=FALSE;
+      PutClientMsg("Cannot get user info from http basic<br>Check http basic setting in .htaccess and other");
+      exit(0);
+    }
+  }
+  return ret;
+}
+
 /********************************************/
 /* get data related to cookie from client  */
 /********************************************/
-int getCookieData(char *userid, char *clientAddr4, int *duration, int *durationEntered, char *language)
+int getCookieData(char *userid, char *clientAddr4, int *duration, int *durationEntered, char *language, char* closeTime)
 {
   char cookie[SIDMAXLN]=""; /* md5 session key from cookie */
   char useridInCookie[USERMAXLN]=""; /* userid from cookie */
@@ -204,7 +253,7 @@ int getCookieData(char *userid, char *clientAddr4, int *duration, int *durationE
 
   /* get related info from DB */ 
   if(!GetSessionInfoFromDb(cookie, userid, clientAddr4, macAddrInDb, 
-			   duration, durationEntered, language)) return FALSE;
+			   duration, durationEntered, language, closeTime)) return FALSE;
     
   /* if userid is changed, cookie auth is failed */
   if(strcmp(useridInCookie, userid)!=0) return FALSE;
@@ -238,7 +287,7 @@ int getAuthCookie(char *cookie, char *userid){
 
  /* if exist cookie, copy it to work area */
   if(isNull(getenv("HTTP_COOKIE"))) return FALSE;
-  strncpy(content, getenv("HTTP_COOKIE"), BUFFMAXLN);
+  strlcpy(content, getenv("HTTP_COOKIE"), BUFFMAXLN);
   ptr=content;
 
   /* search 'OpengateAuth' cookie string (terminated by ; or \0) */
@@ -256,9 +305,9 @@ int getAuthCookie(char *cookie, char *userid){
     split(ptr, name, value, next);
 
     if(strstr(name[0], COOKIENAME)!=NULL){
-      strncpy(cookie, value[0], SIDMAXLN);
+      strlcpy(cookie, value[0], SIDMAXLN);
     }else if(strstr(name[0], "Userid")!=NULL){
-      strncpy(userid, value[0], USERMAXLN);
+      strlcpy(userid, value[0], USERMAXLN);
     }
     ptr=next[0];
   }
@@ -298,7 +347,7 @@ void putClientDeny(char *clientAddr4, char *language)
 	   GetConfValue("AuthCgi"));
 
   /* create opengate dir */
-  snprintf(opengateDir, BUFFMAXLN, GetConfValue("OpengateDir"));
+  snprintf(opengateDir, BUFFMAXLN, "%s", GetConfValue("OpengateDir"));
 
   /* create encoded addr4 */
   if(AddrEncode(encodeAddr, clientAddr4)==1){
@@ -307,9 +356,9 @@ void putClientDeny(char *clientAddr4, char *language)
 
   /* protocol is ssl or non-ssl */
   if(strcmp(getenv("SERVER_PORT"),GetServicePortStr("https"))==0){
-    strncpy(protocol, "https",WORDMAXLN);
+    strlcpy(protocol, "https",WORDMAXLN);
   }else{
-    strncpy(protocol, "http",WORDMAXLN);
+    strlcpy(protocol, "http",WORDMAXLN);
   }
 
   /* document path to deny doc */
@@ -317,13 +366,23 @@ void putClientDeny(char *clientAddr4, char *language)
 	    GetConfValue("OpengateDir"),language,GetConfValue("DenyDoc"));
 
   /* replace keyword and send out the file */
-  printf("Content-type: text/html\r\n\r\n");
+  printf("Content-type: text/html\r\n");
+
+  /* if cookie auth is enabled, delete the cookie */
+  if( (*GetConfValue("EnableCookieAuth")!='0') ){
+    printf("Set-Cookie: %s=;expires=Thu, 01-Jan-1970 00:00:00 GMT;path=/;\r\n", COOKIENAME);
+  }
+
+  /* end of html header */
+  printf("\r\n");
+
+  /* send denydoc content */
   HtmlTemplate(denydoc, keys);
 
   return;
 }
 /*********************************************/
-/* deny message to the client            */
+/* put retry message to the client            */
 /*********************************************/
 void putClientRetry(char *language)
 {
@@ -344,10 +403,10 @@ void putClientRetry(char *language)
     };
 
   /* create opengate Dir */
-  snprintf(opengateDir, BUFFMAXLN, GetConfValue("OpengateDir"));
+  snprintf(opengateDir, BUFFMAXLN, "%s", GetConfValue("OpengateDir"));
  
   /* create external URL string */
-  strncpy(externalUrl, GetConfValue("ExternalUrl"), BUFFMAXLN);
+  strlcpy(externalUrl, GetConfValue("ExternalUrl"), BUFFMAXLN);
 
   /* create authcgi URL string */
   snprintf(authCgiUrl, BUFFMAXLN, "%s%s%s/%s",
@@ -401,6 +460,8 @@ void putClientAccept(char *userid, char *sessionId, int port, int pid, char *cli
   char *opengateDir=GetConfValue("OpengateDir");
   char *opengateServerName=GetConfValue("OpengateServerName");
   int startPageType=atoi(GetConfValue("StartPage/Type"));
+  char useridshort[USERMAXLN];
+  char extraId[USERMAXLN];
 
   /* select proper accept doc */
   switch(toupper(*GetConfValue("WatchMode"))){
@@ -424,6 +485,9 @@ void putClientAccept(char *userid, char *sessionId, int port, int pid, char *cli
   /* if positive value is set in duration, TIME watch mode is selected */
   if(durationEntered) pAcceptDoc=GetConfValue("AcceptDocTime");
 
+  /* split id to display short format of userid */
+  SplitId(userid, useridshort, extraId);
+
   /* create path to acceptdoc */
   snprintf(acceptDocPath, BUFFMAXLN, "%s%s/%s/%s",GetConfValue("DocumentRoot"),
 	 GetConfValue("OpengateDir"),language,pAcceptDoc);
@@ -447,7 +511,8 @@ void putClientAccept(char *userid, char *sessionId, int port, int pid, char *cli
   snprintf(portStr, WORDMAXLN, "%d", port);
 
   /* create duration string (duration=sec, display value=min) */
-  snprintf(durationStr, WORDMAXLN, "%d", duration/60);
+  if(duration<60) snprintf(durationStr, WORDMAXLN, "%.2f", duration/60.);
+  else snprintf(durationStr, WORDMAXLN, "%d", duration/60);
 
   /* open acceptdoc */
   if((fp=fopen(acceptDocPath, "r"))==NULL){
@@ -486,12 +551,13 @@ void putClientAccept(char *userid, char *sessionId, int port, int pid, char *cli
     htmlReplace(buff, "%%OPENGATEDIR%%", opengateDir);
     htmlReplace(buff, "%%OPENGATEPORT%%", portStr);
     htmlReplace(buff, "%%DURATION%%", durationStr);
-    htmlReplace(buff, "%%USERID%%", userid);
+    htmlReplace(buff, "%%USERID%%", useridshort);
     htmlReplace(buff, "%%SESSIONID%%", sessionId);
     htmlReplace(buff, "%%LANGUAGE%%", language);
     htmlReplace(buff, "%%TERMINATEURL%%", terminateUrl);
     htmlReplace(buff, "%%HTTPKEEPURL%%", httpkeepUrl);
     htmlReplace(buff, "%%STARTURL%%", startPageUrl);
+    htmlReplace(buff, "%%REDIRECTEDURL%%", redirectedUrl);
     
     /* write out */
     printf("%s",buff);
@@ -513,8 +579,9 @@ int isHttpWatchEnableClient(void)
   if(strcmp(getenv("SERVER_PROTOCOL"),"HTTP/1.0")==0) return FALSE;
 
   /* some user agent does not support long HTTP Keep-Alive */
+  /* last param 0 means case insensitive */
   if(RegExMatch(getenv("HTTP_USER_AGENT"),
-		 GetConfValue("HttpWatch/SkipAgentPattern"))) return FALSE;
+		 GetConfValue("HttpWatch/SkipAgentPattern"),0)) return FALSE;
 
   return TRUE;
 }
@@ -587,7 +654,7 @@ int checkReferer(void)
 {
   char url[BUFFMAXLN]="";
   if(getenv("HTTP_REFERER")!=NULL){
-    strncpy(url,getenv("HTTP_REFERER"),BUFFMAXLN);
+    strlcpy(url,getenv("HTTP_REFERER"),BUFFMAXLN);
     if(strstr(url,GetConfValue("OpengateServerName"))==NULL){
       return FALSE;
     }
@@ -595,6 +662,21 @@ int checkReferer(void)
   return TRUE;
 }
 
+/***************************************************/
+/* trim duration value between zero to max in conf */
+/***************************************************/
+int trimDuration(int duration, int durationEntered){
+
+  /* if no entry, set default. if huge value, set upper-limit */
+  if(!durationEntered){
+    duration = atoi(GetConfValue("Duration/Default"));
+  }else{
+    int durmax=atoi(GetConfValue("Duration/Max"));
+    if(duration > durmax) duration=durmax;
+  }
+  return duration;
+}
+
 /*******************************/
 /*******************************/
 void GetClientAddr(char *clientAddr)
@@ -615,6 +697,14 @@ int GetPostData(char *userid, char *password, char *clientAddr4, int *durationPt
   return ret;
 }
 
+int GetUserIdFromEnv(char *userid){
+  int ret;
+  if(debug>1) err_msg("DEBUG:=>getUserIdFromEnv(%s)",userid);
+  ret=getUserIdFromEnv(userid);
+  if(debug>1) err_msg("DEBUG:%d<=getUserIdFromEnv(%s)",ret,userid);
+  return ret;
+}
+
 void PutClientAccept(char *userid, char *sessionId, int port, int pid, char *clientAddr4, char *clientAddr6, int ipStatus, int duration, int durationEntered, char *language, char *cookie, int cookieAuth, char *redirectedUrl)
 {
   if(debug>1) err_msg("DEBUG:=>putClientAccept(%s,%s,%d,%d,%s,%s,%d,%d,%d,%s,%s,%d,%s)",userid,sessionId,port,pid,clientAddr4,clientAddr6,ipStatus, duration, durationEntered, language, cookie, cookieAuth, redirectedUrl);
@@ -662,11 +752,22 @@ int GetAuthCookie(char *cookie, char *userid){
   return ret;
 }
 
-int GetCookieData(char *userid, char *clientAddr4, int *duration, int *durationEntered, char *language){
+int GetCookieData(char *userid, char *clientAddr4, int *duration, int *durationEntered, char *language, char* closeTime){
   int ret;
 
   if(debug>1) err_msg("DEBUG:=>getCookieData( )");
-  ret=getCookieData(userid,clientAddr4,duration,durationEntered,language);
-  if(debug>1) err_msg("DEBUG:%d<=getCookieData(%s,passwd,%s,%d,%d,%s)",ret,userid,clientAddr4,*duration,*durationEntered,language);
+  ret=getCookieData(userid,clientAddr4,duration,durationEntered,
+		    language,closeTime);
+  if(debug>1) err_msg("DEBUG:%d<=getCookieData(%s,passwd,%s,%d,%d,%s,%s)",ret,userid,clientAddr4,*duration,*durationEntered,language,closeTime);
+  return ret;
+}
+
+int TrimDuration(int duration, int durationEntered){
+  int ret;
+
+  if(debug>1) err_msg("DEBUG:=>trimDuration(%d,%d)", 
+		      duration,durationEntered);
+  ret=trimDuration(duration,durationEntered);
+  if(debug>1) err_msg("DEBUG:%d<=trimDuration( )",ret);
   return ret;
 }
