@@ -86,7 +86,7 @@ int addAlarm(char *name, int timeout, int preceding, Sigfunc *func)
   }
 
   /* set the item info */
-  strncpy(pNew->name, name, WORDMAXLN);
+  strlcpy(pNew->name, name, WORDMAXLN);
   pNew->timeout = timeout;
   pNew->unixtime = time(NULL) + timeout;
   pNew->preceding = preceding;
