@@ -30,10 +30,6 @@ extern char ruleNumber6[WORDMAXLN];  /* ip6fw rule number in string form */
 struct clientAddr *pClientAddr = NULL;
 int ipStatus;
 
-void RemoveOverlapRule(int overlapRule4, int overlapRule6);
-void KillOverlapProcess(int overlapRule4, int overlapRule6, 
-			char *clientAddr4, char *clientAddr6);
-
 /**************************************/
 /* Open client gate for IPv4 and IPv6 */
 /**************************************/
@@ -41,7 +37,6 @@ int openClientGate(char *clientAddr4, char *macAddr4, char *clientAddr6, char *m
 {
   int ret;
   struct clientAddr *pLastClientAddr=NULL;
-  int overlapRule4=0, overlapRule6=0;
 
   switch(ipStatus){
 
@@ -49,7 +44,8 @@ int openClientGate(char *clientAddr4, char *macAddr4, char *clientAddr6, char *m
   case IPV46DUAL:
     
     /*** at first, open IPv4 gate *****/
-    if((ret=OpenClientGate4(clientAddr4, userid, macAddr4, userProperty))==0){
+    ret=OpenClientGate4(clientAddr4, userid, macAddr4, userProperty);
+    if(ret==0){ /* normally open */
 
       /* open success */
       err_msg("OPEN: user %s from %s at %s", userid, clientAddr4, macAddr4);
@@ -58,18 +54,21 @@ int openClientGate(char *clientAddr4, char *macAddr4, char *clientAddr6, char *m
       pLastClientAddr=pClientAddr=CreateAddrListItem(clientAddr4,ruleNumber4,IPV4);
     }
 
-    /* open fail */
-    else if(ret==-2 || ret==-1 || ret>0){
+    /* dupliacted rules */
+    else  if(ret<-2){
+      return FALSE;
+    }
+
+    /* some failure */
+    else {
       PutClientMsg("Error: Please contact to the administrator");
       return FALSE;
     }
 
-    /* other case (ret<-2) means overlapped rule */
-    else overlapRule4=-ret;
-
 
     /*** next, open IPv6 gate *****/
-    if((ret=OpenClientGate6(clientAddr6, userid, macAddr6, userProperty))==0){
+    ret=OpenClientGate6(clientAddr6, userid, macAddr6, userProperty);
+    if(ret==0){ /* normally open */
 
       /* open success */
       err_msg("OPEN: user %s from %s at %s", userid, clientAddr6, macAddr6);
@@ -77,7 +76,13 @@ int openClientGate(char *clientAddr4, char *macAddr4, char *clientAddr6, char *m
       /* add the address info to list */
       pLastClientAddr->next = CreateAddrListItem(clientAddr6,ruleNumber6,IPV6);
     }
-    /* open fail */
+
+    /* save dupliacted rule */
+    else if(ret<-2){
+      return FALSE;
+    }
+
+    /* some failure */
     else{
       /* remove previously registered ipv4 rule from list */ 
       if(pClientAddr!=NULL && pClientAddr->ipType==IPV4){
@@ -85,23 +90,18 @@ int openClientGate(char *clientAddr4, char *macAddr4, char *clientAddr6, char *m
 	free(pClientAddr);
 	pClientAddr=NULL;
       }
-      /* some system error */
-      if(ret==-2 || ret==-1 || ret>0){
-	PutClientMsg("Error: Please contact to the administrator");
-	return FALSE;
-      }
-      
-      /* other case (ret<-2) means overlapped rule */
-      else overlapRule6=-ret;
+      PutClientMsg("Error: Please contact to the administrator");
+      return FALSE;
     }
-      break;  /* case IPV46DUAL end */
+    break;  /* case IPV46DUAL end */
 
 
     /***** if client have IPv4 only, do below *******/
   case IPV4ONLY:
 
     /* open IPv4 gate */
-    if((ret=OpenClientGate4(clientAddr4, userid, macAddr4, userProperty))==0){
+    ret=OpenClientGate4(clientAddr4, userid, macAddr4, userProperty);
+    if(ret==0){ /* normally open */
 
       /* open success */
       err_msg("OPEN: user %s from %s at %s", userid, clientAddr4, macAddr4);
@@ -111,22 +111,24 @@ int openClientGate(char *clientAddr4, char *macAddr4, char *clientAddr6, char *m
 	=CreateAddrListItem(clientAddr4,ruleNumber4,IPV4);
     }
 
-    /* open fail */
-    else if(ret==-2 || ret==-1 || ret>0){
-      PutClientMsg("Error: Please contact to the administrator");
+    /* dupliacted rule */
+    else if(ret<-2){
       return FALSE;
     }
 
-    /* other case (ret<-2) means overlapped rule */
-    else overlapRule4=-ret;
-
+    /* some failure */
+    else{
+      PutClientMsg("Error: Please contact to the administrator");
+      return FALSE;
+    }
     break; /* case IPV4ONLY end */
     
 
     /***** if client have IPv6 only, do below ******/
   case IPV6ONLY:
     /* open IPv6 gate */
-    if((ret=OpenClientGate6(clientAddr6, userid, macAddr6, userProperty))==0){
+    ret=OpenClientGate6(clientAddr6, userid, macAddr6, userProperty);
+    if(ret==0){ /* normally open */
 
       /* open success */
       err_msg("OPEN: user %s from %s at %s", userid, clientAddr6, macAddr6);
@@ -134,126 +136,50 @@ int openClientGate(char *clientAddr4, char *macAddr4, char *clientAddr6, char *m
       /* create new address list. head and tail pointer point the same item. */
        pLastClientAddr=pClientAddr
 	 =CreateAddrListItem(clientAddr6,ruleNumber6,IPV6);
+    }
+
+    /* dupliacted rule */
+    else if(ret<-2){
+      return FALSE;
      }
 
     /* open fail */
-    else if(ret==-2 || ret==-1 || ret>0){
+    else{
       PutClientMsg("Error: Please contact to the administrator");
       return FALSE;
     }
-    
-    /* other case (ret<-2) means overlapped rule */
-    else overlapRule6=-ret;
-
     break; /* case IPv6ONLY end */
 
   default:
     err_msg("ERR at %s#%d: abnormal IP versions %d",__FILE__,__LINE__,ipStatus);
+    return FALSE;
 
   } /* switch end */
   
-
-  /* if overlapped rules exist, remove them */
-  if(overlapRule4>0 || overlapRule6>0){
-
-    /* to cope to modification of IPv4 addr in Dual stack(send as http param)*/ 
-    if(ipStatus==IPV46DUAL && IsSameMacAddr(macAddr4, macAddr6)==FALSE){
-      err_msg("ERR at %s#%d: arp and ndp return different mac addr",
-	      __FILE__,__LINE__);
-      PutClientMsg("Error. Please End Web and Retry. ");
-    
-    }else{
-
-      /* ipfw rule is deleted and the process is killed */
-      RemoveOverlapRule(overlapRule4,overlapRule6);
-      KillOverlapProcess(overlapRule4,overlapRule6,clientAddr4,clientAddr6);
-
-      err_msg("INFO: user %s forces to close overlapped client", userid);
-      PutClientMsg("Network is closed. Please End Web and Retry. ");
-    }
-    return FALSE;
-  }
-
   return TRUE;
 }
 
 /*************************************************/
-/* remove overlapped firewall rule for same addr */
+/* remove duplicated firewall rule for same addr */
 /*************************************************/
-void removeOverlapRule(int overlapRule4, int overlapRule6)
+void removeDuplicateRule(int duplicateRule4, int duplicateRule6)
 {
   char ruleNumStr[WORDMAXLN];
 
   /* if ipfw rule exist, delete it */
-  if(overlapRule4>0){
-    snprintf(ruleNumStr,WORDMAXLN,"%d",overlapRule4);
+  if(duplicateRule4>0){
+    snprintf(ruleNumStr,WORDMAXLN,"%d",duplicateRule4);
     DelIpfwRule(ruleNumStr);
   }
 
   /* if ip6fw rule exist, delete it */
-  if(overlapRule6>0){
-    snprintf(ruleNumStr,WORDMAXLN,"%d",overlapRule6);
+  if(duplicateRule6>0){
+    snprintf(ruleNumStr,WORDMAXLN,"%d",duplicateRule6);
     DelIp6fwRule(ruleNumStr);
   }
 
 }
 
-/*************************************************/
-/* kill overlapped process                       */
-/*************************************************/
-void killOverlapProcess(int overlapRule4, int overlapRule6, char *clientAddr4, char *clientAddr6){
-
-  FILE *fpipe;
-  char buff[BUFFMAXLN];
-  char cgiNamePattern[WORDMAXLN];
-  char addr4Pattern[ADDRMAXLN]="*dummy*";
-  char addr6Pattern[ADDRMAXLN]="*dummy*";
-  int procNumber=0;
-  char procNumStr[WORDMAXLN];
-
-  /* make cgi name pattern in ps title */
-  snprintf(cgiNamePattern,WORDMAXLN," %s: ", GetConfValue("MainCgi"));
-  
-  /* make cgi param pattern in ps title */
-  if(overlapRule4>0){
-    snprintf(addr4Pattern,ADDRMAXLN,"[%s(%d)]", clientAddr4, overlapRule4);
-  }
-  if(overlapRule6>0){
-    snprintf(addr6Pattern,ADDRMAXLN,"[%s(%d)]", clientAddr6, overlapRule6);
-  }
- 
-  /* search process corresponding to the overlap rule */
-  /* opengate.cgi process shows the following title line for ps command */
-  /* "123 ... opengatesrv.cgi: ..,[123.4.5.6(10000)],[2001:1:1:1::1(10000)]..." */
-
-  /* exec "ps -axww" */
-  if((fpipe=Popenl(1, "r", GetConfValue("PsPath"), "-axww", (char *)0)) == NULL){ 
-    err_msg("ERR at %s#%d: exec ps ax error",__FILE__,__LINE__);
-  }
-    
-  /* read "ps -axww" output as follows */
-  while(fgets(buff, BUFFMAXLN, fpipe)!=NULL){
-
-    /* if opengatesrv cgi corresponding to overlap rule */
-    if(strstr(buff, cgiNamePattern)!=NULL &&
-       ( strstr(buff, addr4Pattern)!=NULL ||
-	 strstr(buff, addr6Pattern)!=NULL ) ){
-
-      /* get the process number */
-      sscanf(buff, "%d", &procNumber);
-
-      /* kill the process */
-      snprintf(procNumStr, WORDMAXLN, "%d", procNumber);
-      if(Systeml(1, "kill", procNumStr, (char *)0) != 0){
-	err_msg("ERR at %s#%d: exec kill error",__FILE__,__LINE__);
-      }
-    }
-  }
-	   
-  /* close pipe */
-  Pclose(fpipe);
-}  
-
 /**************************/
 /* make address list      */
 /**************************/
@@ -266,8 +192,8 @@ struct clientAddr *createAddrListItem(char* ipAddr, char *ruleNumber, int ipType
     PutClientMsg("Error: Please contact to the administrator");
     exit(1);
   }
-  strncpy(pNew->ipAddr,ipAddr,ADDRMAXLN);
-  strncpy(pNew->ruleNumber,ruleNumber,WORDMAXLN);
+  strlcpy(pNew->ipAddr,ipAddr,ADDRMAXLN);
+  strlcpy(pNew->ruleNumber,ruleNumber,WORDMAXLN);
   pNew->ipType = ipType;
   pNew->timeIn = time(NULL);
   pNew->activeStatus=TRUE;
@@ -350,22 +276,13 @@ int CheckIpVersions(char *clientAddr4, char *clientAddr6)
   return ret;
 }
 
-void RemoveOverlapRule(int overlapRule4, int overlapRule6){
+void RemoveDuplicateRule(int duplicateRule4, int duplicateRule6){
 
-  if(debug>1) err_msg("DEBUG:=>removeOverlapRule(%d,%d)",overlapRule4,overlapRule6);
-  removeOverlapRule(overlapRule4,overlapRule6);
-  if(debug>1) err_msg("DEBUG:<=removeOverlapRule( )");
+  if(debug>1) err_msg("DEBUG:=>removeDuplicateRule(%d,%d)",duplicateRule4,duplicateRule6);
+  removeDuplicateRule(duplicateRule4,duplicateRule6);
+  if(debug>1) err_msg("DEBUG:<=removeDuplicateRule( )");
 }
 
-void KillOverlapProcess(int overlapRule4, int overlapRule6, 
-		       char *clientAddr4, char *clientAddr6){
-  if(debug>1) err_msg("DEBUG:=>killOverlapProcess(%d,%d,%s,%s)", 
-		    overlapRule4,overlapRule6,clientAddr4,clientAddr6);
-  killOverlapProcess(overlapRule4,overlapRule6,clientAddr4,clientAddr6);
-  if(debug>1) err_msg("DEBUG:<=killOverlapProcess( )");
-}
-
-
 int GetPacketCount(struct clientAddr *pClientAddr)
 {
   int ret;
