@@ -50,6 +50,7 @@ int  main(int argc, char **argv)
   char *value[1];
   char *next[1];
   char *ptr;
+  char *authProtocol;
 
   /* keyword pairs */
   /*  the left key is replaced by the right value */
@@ -67,6 +68,12 @@ int  main(int argc, char **argv)
       {"",""}  /* DON'T REMOVE THIS LINE */
     };
 
+  /* if this is executed in shell with '-v' option, show make dir */  
+  if(argc>1 && strcmp(argv[1],"-v")==0){
+    printf("makedir: %s\n", MAKEDIR);
+    exit(0);
+  }
+
   /* prepare config file */
   if(OpenConfFile()==-1) return 0;
  
@@ -79,8 +86,8 @@ int  main(int argc, char **argv)
 
   if(debug>1) err_msg("DEBUG: started");
 
-  snprintf(opengateServerName,BUFFMAXLN,GetConfValue("OpengateServerName"));
-  snprintf(opengateDir,BUFFMAXLN,GetConfValue("OpengateDir"));
+  snprintf(opengateServerName,BUFFMAXLN,"%s",GetConfValue("OpengateServerName"));
+  snprintf(opengateDir,BUFFMAXLN,"%s",GetConfValue("OpengateDir"));
 
   /* create URL string */
   snprintf(authCgiUrl, BUFFMAXLN, "%s%s%s/%s",
@@ -102,7 +109,7 @@ int  main(int argc, char **argv)
   if(!isNull(getenv("QUERY_STRING"))){
 
     /* get html access parameter string (attached at the end of url) */
-    strncpy(paramString, getenv("QUERY_STRING"), BUFFMAXLN);
+    strlcpy(paramString, getenv("QUERY_STRING"), BUFFMAXLN);
   }
 
   /* split language and address in paramString
@@ -124,14 +131,14 @@ int  main(int argc, char **argv)
   if(isNull(pAddr4)){
     clientAddr[0]='\0';
   }else{
-    strncpy(clientAddr, pAddr4, ADDRMAXLN);
+    strlcpy(clientAddr, pAddr4, ADDRMAXLN);
   }
 
   /* get language and check its correctness */
   if(!isNull(pLang) && strstr(GetConfValue("HtmlLangs"), pLang)!=NULL){
 
     /* if corrrect, set it */
-    strncpy(lang, pLang, WORDMAXLN);
+    strlcpy(lang, pLang, WORDMAXLN);
   }else{
 
     /* if not correct, get default language at the top of lang list */
@@ -139,7 +146,7 @@ int  main(int argc, char **argv)
   }
 
   /* copy redirected URL */
-  strncpy(redirectedUrl, pUrl, BUFFMAXLN);
+  strlcpy(redirectedUrl, pUrl, BUFFMAXLN);
 
   /* send out header */
   printf("Content-Type: text/html\r\n\r\n");
@@ -148,19 +155,21 @@ int  main(int argc, char **argv)
   sprintf(htmlFile, "%s%s/%s/", GetConfValue("DocumentRoot"),
 	  opengateDir, lang);
 
-  /*** if http-cookie for opengate authentication exist */
-  /* and cookie auth is enabled, */
+  /*** if http-cookie exists or auth by shibboleth/httpbasic */
   /* skip authenticaion */
-  if( (*GetConfValue("EnableCookieAuth")!='0')
-     && GetAuthCookie(cookie,userid)){
-    strncat(htmlFile, GetConfValue("SkipAuthDoc"), BUFFMAXLN);
+  authProtocol=GetConfValue("AuthServer/Protocol");
+  if(((*GetConfValue("EnableCookieAuth")!='0') && GetAuthCookie(cookie,userid))
+     || strcmp(authProtocol, "shibboleth")==0 
+     || strcmp(authProtocol, "httpbasic")==0 ){
+
+    strlcat(htmlFile, GetConfValue("SkipAuthDoc"), BUFFMAXLN);
 
     /* ssl or non-ssl protocol  */
     if(!isNull(getenv("SERVER_PORT"))
        && strcmp(getenv("SERVER_PORT"),GetServicePortStr("https"))==0) {
-      strncpy(protocol, "https", WORDMAXLN);
+      strlcpy(protocol, "https", WORDMAXLN);
     }else{
-      strncpy(protocol, "http", WORDMAXLN);
+      strlcpy(protocol, "http", WORDMAXLN);
     }
   }
   
@@ -169,9 +178,9 @@ int  main(int argc, char **argv)
     /* ssl or non-ssl file */
     if(!isNull(getenv("SERVER_PORT"))
        && strcmp(getenv("SERVER_PORT"),GetServicePortStr("https"))==0) {
-      strncat(htmlFile, GetConfValue("AuthDocSsl"), BUFFMAXLN);
+      strlcat(htmlFile, GetConfValue("AuthDocSsl"), BUFFMAXLN);
     }else{
-      strncat(htmlFile, GetConfValue("AuthDoc"), BUFFMAXLN);
+      strlcat(htmlFile, GetConfValue("AuthDoc"), BUFFMAXLN);
     }
   }
 
