@@ -31,7 +31,7 @@ void ReadHttpHeaders(void);
 void SendReplyToGetHello(void);
 void SendHttpKeepPage(char *userid, char *sessionId, char *language, int port, int cookieAuth, char *redirectedUrl);
 int SelectAccept(void);
-void AcceptHttpReConnect(void);
+void AcceptHttpReConnect(int timeout);
 int MacAddrCheck(int ipStatus,char *clientAddr4, char *clientAddr6, char *macAddr4, char *macAddr6);
 int IsRightKey(char *pNowKey, char *sessionId);
 void SendHttpReply(char *reply);
@@ -42,6 +42,9 @@ void OnCheckBasicAlarm(int signo);
 void OnCheckHttpAlarm(int signo);
 void OnReadWaitAlarm(int signo);
 void OnAjaxWaitAlarm(int signo);
+void CheckBasic(void);
+void CheckHttp(void);
+
 
 extern char ruleNumber4[WORDMAXLN];  /* ipfw rule number in string form  */
 extern char ruleNumber6[WORDMAXLN];  /* ip6fw rule number in string form */
@@ -52,6 +55,7 @@ int listenfd[2]; /* file descriptor for listen port */
 int hasSock6=TRUE; /* can get the socket for IPv6 */  
 int connfd;   /* file descriptor for connection port */
 int connectMode = NOCONNECT; /* the TCP connection mode */
+char previousHello[BUFFMAXLN];  /* save previous hello request string */
 
 struct AlarmArg{          /* arguments used in on-alarm functions */
   struct clientAddr *pClientAddr;
@@ -71,6 +75,9 @@ int readHelloTime=0;  /* the time of reading hello */
 int sendHelloTime=0;  /* the time of sending hello */
 int noReplyCount=0; /* count up the no reply to hello message */
 
+int checkBasicAlarmRinged=FALSE;
+int checkHttpAlarmRinged=FALSE;
+
 
 /***************************************/
 /* get temp listen port of this server */
@@ -179,16 +186,22 @@ int waitClientConnect(char *userid, char *userProperty, char *sessionId, char *c
     /* start alarms */
     EnableAlarm();
 
-    /* connection wait. retry at abnormal  */
-    if((connfd = SelectAccept()) < 0 ){
+    /* connection wait */
+    connfd = SelectAccept();
+
+    /* stop alarms*/
+    DisableAlarm();
+
+    /* if abnormal accept check interupt */
+    if(connfd<0){
+      
+      /* periodic check */
+      if(checkBasicAlarmRinged) CheckBasic();
+
       Close(connfd);
       continue;
     }
 
-    /* at normal connection */
-    /* stop alarm interupt between checking */
-    DisableAlarm();
-    
     /* is it from the correct client addr(the check is skipped for IPv6) */
     if(ipType==IPV4 && ipStatus!=IPV6ONLY){
       GetPeerAddr(connfd, connectAddr);
@@ -284,7 +297,8 @@ int waitClientConnect(char *userid, char *userProperty, char *sessionId, char *c
 /****************************/
 int selectAccept(void)
 {
-  int connfd = -1;
+  int connfd1 = -1;
+  int connfd2 = -1;
   int smax;                         /* select max descliptor */
   fd_set rfd0;                      /* fd_set for select */
   int n;                            /* counter */
@@ -303,29 +317,34 @@ int selectAccept(void)
     smax=listenfd[0]+1;
   }
 
+  /* wait connection */
   if((n = select(smax, &rfd0, NULL, NULL, NULL)) > 0){
-    /* wait connection */
 
+    /* connect by ipv4 */
     if(FD_ISSET(listenfd[0], &rfd0)){
-
-      if((connfd=accept(listenfd[0],(struct sockaddr *)&cliaddr, &len))>=0){
-
-	/* connect by ipv4 */
+      if((connfd1=accept(listenfd[0],(struct sockaddr *)&cliaddr, &len))>=0){
 	ipType=IPV4;
       }
     }
+
+    /* connect by ipv6 */
     if(hasSock6){
       if(FD_ISSET(listenfd[1], &rfd0)){
-
-	if((connfd=accept(listenfd[1],(struct sockaddr *)&cliaddr, &len))>=0){
-
-	  /* connect by ipv6 */
+	if((connfd2=accept(listenfd[1],(struct sockaddr *)&cliaddr, &len))>=0){
 	  ipType=IPV6;
 	}
       }
     }
   }
-  return connfd;
+  
+  if(connfd1>=0 && connfd2>=0){
+    Close(connfd2);
+    return connfd1;
+  }
+  else if(connfd1>=0 && connfd2<0) return connfd1;
+  else if(connfd1<0 && connfd2>=0) return connfd2;
+
+  return -1;
 }
 
 /***************************************/
@@ -353,14 +372,25 @@ void onAjaxWaitAlarm(int signo)
   connectMode=NOCONNECT;
 }
 
- /***************************************/
+/***************************************/
 /* called at periodic alive basic check */
 /***************************************/
 void onCheckBasicAlarm(int signo)
+{
+  checkBasicAlarmRinged=TRUE;
+}
+
+/***************************************/
+/* called at periodic alive basic check */
+/***************************************/
+void checkBasic(void)
 {
   static int packetCountPrev=0;  /* packet count at previous check */
   int packetCountNow=0;   /* packet count at now */
-  static   int noPacketPeriod=0; /* no packet period count in check loop */
+  static int noPacketIntervalCount=0; /* no packet count in check loop */
+  static int firstCheck=TRUE;
+  int duplicateRule4,duplicateRule6;
+  int otherPid;
 
   /* search new IPv6 addresses */
   ScanNdpEntry(alarmArg.pClientAddr, alarmArg.userid,
@@ -377,20 +407,48 @@ void onCheckBasicAlarm(int signo)
   /* packet flow check */
   packetCountNow=GetPacketCount(alarmArg.pClientAddr);
   if(packetCountNow==packetCountPrev){  /* no packet between checks */
-    noPacketPeriod++;
+    noPacketIntervalCount++;
   }else{
-    noPacketPeriod=0;
+    noPacketIntervalCount=0;
     packetCountPrev=packetCountNow;
   }
   
-  if(noPacketPeriod*alarmArg.checkInterval 
+  if(noPacketIntervalCount*alarmArg.checkInterval 
      >= alarmArg.noPacketInterval){
     err_msg("ERR at %s#%d: no packet passed for the client",
 	    __FILE__,__LINE__);
     connectMode=ENDCONNECT;
     return;
   }
+
+  /* duplicated session check */
+  /* this is added for ios behavior */
+  /* (duplication of ios web-auth and user web-auth) */
+  /* if dup, the process in basic mode is ended */
+  if(firstCheck){
+    firstCheck=FALSE;
+    if( FindDuplicateInDbAndClose(alarmArg.clientAddr4, 
+				  &duplicateRule4, &duplicateRule6, &otherPid) ){
+      
+      /* the process exists (ret==0 means the existence) */
+      if(kill(otherPid, 0)==0){ 
+
+	/* remove my rules and return as duplicated */
+	RemoveDuplicateRule(duplicateRule4, duplicateRule6);
+	err_msg("ERR at %s#%d: duplicated ipfw rules for %s are removed",
+		__FILE__,__LINE__, alarmArg.clientAddr4);
+	connectMode=DUPLICATED;
+      }
+
+      /* the process does not exist */
+      else{ 
+	FixProcessEndInDb(otherPid, "NONE");
+      }
+    }
+  }
+
   /* set the alarm for next periodic keep alive check */
+  checkBasicAlarmRinged=FALSE;
   AddAlarm("CheckBasicAlarm", alarmArg.checkInterval, FALSE, OnCheckBasicAlarm);  /* EnableAlarm is called automatically in alarm function */
 }
 
@@ -438,7 +496,7 @@ void getPeerAddr(int sockfd, char *peerAddr)
   if(Getpeername(sockfd, cliaddr, &len)<0) return; /* if error, return */
     
   pAddr=Sock_ntop_host(cliaddr, len);
-  if(pAddr!=NULL) strncpy(peerAddr, pAddr, ADDRMAXLN);
+  if(pAddr!=NULL) strlcpy(peerAddr, pAddr, ADDRMAXLN);
   
   free(cliaddr);
   
@@ -451,6 +509,13 @@ void getPeerAddr(int sockfd, char *peerAddr)
 /***************************************************/
 void sendTerminateReply(void)
 {
+  /* send HTTP headers */
+  Writefmt(connfd,"HTTP/1.1 200 OK\r\n");
+  Writefmt(connfd,"Connection: Close\r\n");
+  Writefmt(connfd,"Content-Type: text/html\r\n");
+  Writefmt(connfd,"Content-Length: 92\r\n");
+  Writefmt(connfd,"\r\n");
+  
   Writefmt(connfd,"<META HTTP-EQUIV=Pragma CONTENT=no-cache>");
   Writefmt(connfd,"<HTML><BODY> Network is closed. </BODY></HTML> \r\n\r\n");
   Close(connfd);
@@ -481,6 +546,7 @@ void waitHttpClose(struct clientAddr *pClientAddr, char *userid, char *userPrope
   alarmArg.userid=userid;
   alarmArg.userProperty=userProperty;
   alarmArg.ipStatus=ipStatus;
+  alarmArg.checkInterval=atoi(GetConfValue("ActiveCheckInterval"));
 
   /* save time to read hello */
   readHelloTime = time(NULL);
@@ -505,10 +571,11 @@ void waitHttpClose(struct clientAddr *pClientAddr, char *userid, char *userPrope
   AddAlarm("checkHttpAlarm", alarmArg.checkInterval, FALSE, OnCheckHttpAlarm);
   EnableAlarm();
   
-  while(1){
+    /* loop unitl end connection */  
+  while(connectMode!=ENDCONNECT){
 
-    /* set default to terminate */
-    connectMode=ENDCONNECT;
+    /* set default to unknown */
+    connectMode=NOCONNECT;
 
     /* read in wait for client request */
     ret=readln(connfd, buff, BUFFMAXLN);
@@ -518,25 +585,45 @@ void waitHttpClose(struct clientAddr *pClientAddr, char *userid, char *userPrope
 
     if(ret>=0){
       /* normal read */
-      connectMode=HTTPCONNECT;
 
       /* read [GET /hello] */
       if(strstr(buff,"GET /hello") == buff){
 	
 	ReadHttpHeaders();
 
-	/* if received key is not match, ignore */
-	if(IsRightKey(buff+strlen("GET /hello-"), sessionId)==FALSE){
+	/* if retry request (having same key as previous) */
+	if(strncmp(buff, previousHello, BUFFMAXLN)==0) {
+
+	  /*SendReplyToGetHello();*/
+	  /* some mulfunction on communication. reset connection */
+	  connectMode=NOCONNECT;
+	  Close(connfd);
+	  AcceptHttpReConnect(atoi(GetConfValue("CommWaitTimeout")));
 	  continue;
 	}
-	
-	/* save time to read hello */
-	readHelloTime = time(NULL);
 
-	/* set the hello wait mode OFF */
-	helloWait=FALSE;
-      }
+	/* save hello request string including keys */	  
+	strlcpy(previousHello, buff, BUFFMAXLN);
+
+	/* if received key is correct, save time */
+	if(IsRightKey(buff+strlen("GET /hello-"), sessionId)){
+	
+	  /* recognize HTTP hello request */
+	  connectMode=HTTPCONNECT;
+	
+	  /* save time to read hello */
+	  readHelloTime = time(NULL);
+
+	  /* set the hello wait mode OFF */
+	  helloWait=FALSE;
+	}
       
+	/* if key is incorrect, terminate */
+	else{
+	  connectMode=ENDCONNECT;
+	}
+      }
+
       /* read other string */
       else{
 	
@@ -545,30 +632,30 @@ void waitHttpClose(struct clientAddr *pClientAddr, char *userid, char *userPrope
 	  ReadHttpHeaders();
 	  SendHttpNotFound();
 	}
-	connectMode=ENDCONNECT;
+	err_msg("ERR at %s#%d: Unknown request",__FILE__,__LINE__);
+	connectMode=NOCONNECT;
 	Close(connfd);
+	AcceptHttpReConnect(atoi(GetConfValue("CommWaitTimeout")));
       }
     }
+
     else{
       /*abnormal read */
       /* some alarm is ringed or connecion is closed */
-      /*  connectionMode might be modified in onAlarm */
 
-      if(connectMode==ENDCONNECT){
+      /* periodic check */
+      if(checkHttpAlarmRinged) CheckHttp();
 
-	/* close the connection */
-	Close(connfd);
+      /* if no connection, retry connect */
+      if(connectMode==NOCONNECT){
 
 	/* wait short time to accept reconnection */
 	/* (to permit occasional disconnection at exchanging hello) */
-	if(helloWait==TRUE &&
-	   (time(NULL)-sendHelloTime)<=atoi(GetConfValue("ReconnectTimeout"))){
-	  AcceptHttpReConnect();
-	}
-
-	/* no reconnection, then terminate */
-	if(connectMode==ENDCONNECT){
-	  break;
+	Close(connfd);
+	if(helloWait==TRUE){
+	  AcceptHttpReConnect(atoi(GetConfValue("ReconnectTimeout")));
+	}else{
+	  AcceptHttpReConnect(atoi(GetConfValue("CommWaitTimeout")));
 	}
       }
     }
@@ -588,15 +675,20 @@ void waitHttpClose(struct clientAddr *pClientAddr, char *userid, char *userPrope
 /*************************/
 /* accept http reconnect */
 /*************************/
-void acceptHttpReConnect(void){
+void acceptHttpReConnect(int timeout){
 
   int startTime; 
 
   startTime=time(NULL);
 
+  /* timeout value should be positive */
+  if(timeout<=0){
+    connectMode=ENDCONNECT;
+    return;
+  }
+
   /* wait a short time */
-  AddAlarm("ReadWaitAlarm",atoi(GetConfValue("ReconnectTimeout")),
-	   TRUE, OnReadWaitAlarm); 
+  AddAlarm("ReadWaitAlarm", timeout, TRUE, OnReadWaitAlarm); 
   EnableAlarm();
 
   /* connection wait */
@@ -604,7 +696,6 @@ void acceptHttpReConnect(void){
 
   /* receive request */
   RemoveAlarm("ReadWaitAlarm");
-
   if(connfd<0){  /* timeout or abnormal */
     connectMode=ENDCONNECT;
   }else{         /* reconnected */
@@ -641,7 +732,7 @@ int isRightKey(char *arg, char *sessionId)
   /* initial value of savedKey is md5(md5(sessionId)+sessionId) */
   if(isNull(savedKey)){
     md5hex(tempbuff, 33, sessionId);
-    strncat(tempbuff, sessionId, BUFFMAXLN);
+    strlcat(tempbuff, sessionId, BUFFMAXLN);
     md5hex(savedKey, 33, tempbuff);
   }
 
@@ -653,19 +744,17 @@ int isRightKey(char *arg, char *sessionId)
   *(pNextKey+32)='\0';
 
   /* make string [nowKey+sessionId] */
-  strncpy(tempbuff, pNowKey, BUFFMAXLN);
-  strncat(tempbuff, sessionId, BUFFMAXLN);
+  strlcpy(tempbuff, pNowKey, BUFFMAXLN);
+  strlcat(tempbuff, sessionId, BUFFMAXLN);
 
   /* compare savedKey and md5(nowKey+sessionId) */
   if(strcmp(savedKey, md5hex(md5work, 33, tempbuff))==0){
 
     /* save nextKey for next check */
-    strncpy(savedKey, pNextKey, 33);
+    strlcpy(savedKey, pNextKey, 33);
     return TRUE;
   }
   else{
-    err_msg("ERR at %s#%d: incorrect client key",
-	    __FILE__,__LINE__);
     return FALSE;
   }
 }
@@ -674,11 +763,19 @@ int isRightKey(char *arg, char *sessionId)
 /* called at periodic http alive check */
 /***************************************/
 void onCheckHttpAlarm(int signo)
+{
+  checkHttpAlarmRinged=TRUE;
+}
+
+/***************************************/
+/* called at periodic http alive check */
+/***************************************/
+void checkHttp(void)
 {
   /* at this timing, hello request might be received */
   /* if in hello wait mode, it is abnormal */
   if(helloWait==TRUE){
-    connectMode=ENDCONNECT;
+    connectMode=NOCONNECT;
     return;
   }
 
@@ -704,6 +801,7 @@ void onCheckHttpAlarm(int signo)
   }
 
   /* set the alarm for next periodic check */
+  checkHttpAlarmRinged=FALSE;
   AddAlarm("CheckHttpAlarm", alarmArg.checkInterval, FALSE, OnCheckHttpAlarm);
   /* EnableAlarm is called automatically in alarm function */
 
@@ -768,7 +866,11 @@ void sendHttpKeepPage(char *userid, char *sessionId, char *language, int port, i
 
   char *opengateServerName=GetConfValue("OpengateServerName");
   char *opengateDir=GetConfValue("OpengateDir");
+  char useridshort[USERMAXLN];
+  char extraId[USERMAXLN];
 
+  /* split id to display short format of userid */
+  SplitId(userid, useridshort, extraId);
 
   /* create path to accept2 doc */
   snprintf(acceptDoc2Url, BUFFMAXLN, 
@@ -839,7 +941,7 @@ void sendHttpKeepPage(char *userid, char *sessionId, char *language, int port, i
     /* replace mark */
     htmlReplace(buff, "%%OPENGATESERVERNAME%%", opengateServerName);
     htmlReplace(buff, "%%HTTPHELLOURL%%", httpHelloUrl);
-    htmlReplace(buff, "%%USERID%%", userid);
+    htmlReplace(buff, "%%USERID%%", useridshort);
     htmlReplace(buff, "%%SESSIONID%%", sessionId);
     htmlReplace(buff, "%%TERMINATEURL%%", terminateUrl);
     htmlReplace(buff, "%%HTTPKEEPJSURL%%", httpkeepJsUrl);
@@ -850,6 +952,7 @@ void sendHttpKeepPage(char *userid, char *sessionId, char *language, int port, i
     htmlReplace(buff, "%%LANGUAGE%%", language);
     htmlReplace(buff, "%%STARTURL%%", startPageUrl);
     htmlReplace(buff, "%%COOKIENAME%%", COOKIENAME);
+    htmlReplace(buff, "%%REDIRECTEDURL%%", redirectedUrl);
 
     /* length of chunk in hex */
     Writefmt(connfd, "%x\r\n", strlen(buff));
@@ -900,6 +1003,8 @@ void sendHttpNotFound(void)
   Writefmt(connfd,"\r\n");
 }
 
+
+
 /***************************************************/
 /***************************************************/
 void GetPeerAddr(int sockfd, char *peerAddr)
@@ -990,6 +1095,13 @@ void OnCheckBasicAlarm(int signo){
   if(debug>1) err_msg("DEBUG:<=onCheckBasicAlarm()");
 }
 
+void CheckBasic(void){
+
+  if(debug>1) err_msg("DEBUG:=>checkBasic()");
+  checkBasic();
+  if(debug>1) err_msg("DEBUG:<=checkBasic()");
+}
+
 void OnCheckHttpAlarm(int signo){
 
   if(debug>1) err_msg("DEBUG:=>onCheckHttpAlarm()");
@@ -997,6 +1109,13 @@ void OnCheckHttpAlarm(int signo){
   if(debug>1) err_msg("DEBUG:<=onCheckHttpAlarm()");
 }
 
+void CheckHttp(void){
+
+  if(debug>1) err_msg("DEBUG:=>checkHttp()");
+  checkHttp();
+  if(debug>1) err_msg("DEBUG:<=checkHttp()");
+}
+
 void OnReadWaitAlarm(int signo){
 
   if(debug>1) err_msg("DEBUG:=>onReadWaitAlarm()");
@@ -1021,9 +1140,9 @@ int SelectAccept(void){
   return ret;
 }
 
-void AcceptHttpReConnect(void){
-  if(debug>1) err_msg("DEBUG:=>acceptHttpReConnect()");
-  acceptHttpReConnect();
+void AcceptHttpReConnect(int timeout){
+  if(debug>1) err_msg("DEBUG:=>acceptHttpReConnect(%d)",timeout);
+  acceptHttpReConnect(timeout);
   if(debug>1) err_msg("DEBUG:<=acceptHttpReConnect()");
 }
 
@@ -1060,3 +1179,4 @@ void SendHttpNotFound(void){
   sendHttpNotFound();
   if(debug>1) err_msg("DEBUG:<=sendHttpNotFound()");
 }
+
